"""
mlvlab.i18n

Sistema de internacionalización para MLV-Lab.
Proporciona soporte para múltiples idiomas con inglés como idioma por defecto.
"""

from .core import I18nManager, i18n

__all__ = ["I18nManager", "i18n"]
